#ifndef BUILDER_BUILDERIMPL_
#define BUILDER_BUILDERIMPL_

#include "Builder.h"
#include "llvm/Support/Casting.h"
#include "mlir/IR/Attributes.h"
#include "mlir/IR/Block.h"
#include "mlir/IR/Builders.h"
#include "mlir/IR/BuiltinOps.h"
#include "mlir/IR/BuiltinTypes.h"
#include "mlir/IR/MLIRContext.h"
#include "mlir/IR/Operation.h"
#include "mlir/IR/Types.h"
#include "mlir/IR/Value.h"

namespace builder {

class Builder::Impl {
 public:
  Impl() : builder_(&context_) {
    module_ = mlir::ModuleOp::create(mlir::UnknownLoc::get(&context_));

    llvm::SmallVector<mlir::Type, 4> arg_types;
    // Create the main function.
    mlir::FunctionType funcType = builder_.getFunctionType(arg_types, {});
    main_func_ = mlir::FuncOp::create(builder_.getUnknownLoc(), "main",
                                      funcType, /* attrs = */ {});

    entry_block_ = main_func_.addEntryBlock();
    builder_.setInsertionPointToStart(entry_block_);
    module_.push_back(main_func_);
  }

  mlir::Location GetLoc() { return builder_.getUnknownLoc(); }
  mlir::OpBuilder GetBuilder() { return builder_; }
  mlir::MLIRContext* GetContext() { return &context_; }

 private:
  mlir::MLIRContext context_;
  mlir::ModuleOp module_;
  mlir::OpBuilder builder_;
  mlir::FuncOp main_func_;
  mlir::Block* entry_block_;
};

}  // namespace builder

#endif